using System;
using System.Collections.Generic;
using System.Linq;

namespace RPGOnline.Remoting.Entities
{
    public class GroupManager
    {
        private readonly SessionContext m_sessionContext;
        private readonly UserManager m_userManager;
        private readonly ChatManager m_chatManager;

        public ChatGroup DefaultChatGroup { get; set; }

        public event EventHandler OnGroupAdded;
        public event EventHandler OnGroupRemoved;
        public event EventHandler OnUserAddedToGroup;
        public event EventHandler OnUserRemovedFromGroup;
        public event EventHandler OnUserInfoChanged;

        public GroupManager(SessionContext sessionContext, UserManager userManager, ChatManager chatManager)
        {
            m_sessionContext = sessionContext;
            m_userManager = userManager;
            m_chatManager = chatManager;
            m_chatManager.OnUserRegistered += (s, e) => AddUser(e.User);
            m_chatManager.OnUserUnregistered += (s, e) => RemoveUser(e.User);
            m_chatManager.OnUserInfoChanged += (s, e) => UpdateUserInfo(e.User);
            m_chatManager.OnGroupAdded += (s, e) => AddGroup(e.ChatGroup);
            m_chatManager.OnUserMovedToGroup += (s, e) => MoveUserToGroup(e.User, e.GroupId);

            DefaultChatGroup = m_sessionContext.MainChatGroup;
        }

        private void UpdateUserInfo(User user)
        {
            ChatGroup chatGroup = FindChatGroupByUser(user);
            if (chatGroup == null) return;
            chatGroup.Users[user.Id] = user;

            if (OnUserInfoChanged != null)
                OnUserInfoChanged(this, EventArgs.Empty);
        }

        public static ChatGroup CreateGroup(string groupName)
        {
            ChatGroup chatGroup = new ChatGroup(groupName, Guid.NewGuid());
            return chatGroup;
        }

        private void AddGroup(ChatGroup chatGroup)
        {
            m_sessionContext.AddGroup(chatGroup);

            if (OnGroupAdded != null)
                OnGroupAdded(this, EventArgs.Empty);
        }

        private void RemoveGroup(ChatGroup chatGroup)
        {
            foreach (User user in chatGroup.Users.Values)
            {
                MoveUserToGroup(user, Guid.Empty);
            }
            m_sessionContext.RemoveGroup(chatGroup.Id);

            if (OnGroupRemoved != null)
                OnGroupRemoved(this, EventArgs.Empty);
        }

        public ChatGroup FindChatGroupByUser(User user)
        {
            return m_sessionContext.Groups.FirstOrDefault(chatGroup => chatGroup.Contains(user.Id));
        }

        private void AddUser(User user)
        {
            m_sessionContext.AddUser(user, m_sessionContext.MainChatGroup);

            if (user.Equals(m_userManager.CurrentUser))
                m_userManager.CurrentUserGroup = m_sessionContext.MainChatGroup;

            if (OnUserAddedToGroup != null)
                OnUserAddedToGroup(this, EventArgs.Empty);
        }

        private void AddUser(User user, ChatGroup chatGroup)
        {
            m_sessionContext.AddUser(user, chatGroup);

            if (OnUserAddedToGroup != null)
                OnUserAddedToGroup(this, EventArgs.Empty);
        }

        private bool RemoveUser(User user)
        {
            ChatGroup chatGroup = FindChatGroupByUser(user);
            if (chatGroup == null)
                return false;

            chatGroup.Remove(user);

            if (OnUserRemovedFromGroup != null)
                OnUserRemovedFromGroup(this, EventArgs.Empty);

            return true;
        }

        private bool MoveUserToGroup(User user, Guid groupId)
        {
            ChatGroup destinationGroup = m_sessionContext.FindGroupById(groupId);
            if (destinationGroup == null)
                return false;

            if (user.Equals(m_userManager.CurrentUser))
                m_userManager.CurrentUserGroup = destinationGroup;

            RemoveUser(user);
            AddUser(user, destinationGroup);
            return true;
        }
    }
}