using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace RPGOnline.Controls.Map.Layers.Lighting
{
    [TypeConverter(typeof(ExpandableObjectConverter))]
    public class Light
    {
        private Map m_map;
        private LightingLayer m_lightingLayer;
        private Point m_lightPosition;
        private float m_oneMeterInPixels;
        private GraphicsPath m_path;
        private PathGradientBrush m_brush;
        private bool m_invalid = true;
        private Point m_position;
        private float m_radius;
        private Color m_color;

        [Browsable(false)]
        public Map Map
        {
            get { return m_map; }
            set
            {
                m_map = value;
                m_oneMeterInPixels = m_map.TileSize.Width / 1.5f;
            }
        }

        [Browsable(false)]
        public LightingLayer LightingLayer
        {
            get { return m_lightingLayer; }
            set { m_lightingLayer = value; }
        }

        protected Light()
        {
            Position = new Point(0, 0);
            Radius = 10;
            Color = Color.White;
        }

        public Light(Map map, LightingLayer lightingLayer)
            : this()
        {
            Map = map;
            m_lightingLayer = lightingLayer;
        }

        private void Update()
        {
            if (!m_invalid)
                return;

            m_map.GetCellCoordsAt(Position, out m_lightPosition);

            Rectangle boundingRectangle = new Rectangle(Position, Size.Empty);
            boundingRectangle.Inflate((int)(m_oneMeterInPixels * Radius), (int)(m_oneMeterInPixels * Radius));

            GraphicsPath path = new GraphicsPath();
            path.FillMode = FillMode.Winding;
            path.AddEllipse(boundingRectangle);

            m_brush = new PathGradientBrush(path);
            m_brush.InterpolationColors = new ColorBlend(2)
            {
                Colors = new[] { Color.Transparent, Color },
                Positions = new float[] { 0, 1 }
            };

            //draw a rectangle around the start point
            m_path = new GraphicsPath(FillMode.Winding);
            m_path.AddRectangle(new Rectangle(m_lightPosition.X * m_map.TileSize.Width,
                                               m_lightPosition.Y * m_map.TileSize.Height,
                                               m_map.TileSize.Width, m_map.TileSize.Height));
            int radInCells = (int)(Radius / 1.5);
            for (int x = (m_lightPosition.X - radInCells); x <= (m_lightPosition.X + radInCells); x++)
            {
                for (int y = (m_lightPosition.Y - radInCells); y <= (m_lightPosition.Y + radInCells); y++)
                {
                    //check to see if those points appear within the circle
                    if ((Math.Pow(m_lightPosition.X - x, 2) + Math.Pow(m_lightPosition.Y - y, 2)) < (Math.Pow(radInCells, 2)))
                    {
                        //draw a Bresenham line between the start point and the calculated point
                        foreach (Point point in line(m_lightPosition, new Point(x, y)))
                        {
                            Rectangle rectangle = new Rectangle(point.X * m_map.TileSize.Width, point.Y * m_map.TileSize.Height,
                                                                m_map.TileSize.Width, m_map.TileSize.Height);
                            m_path.AddRectangle(rectangle);
                        }
                    }
                }
            }
        }

        public Point Position
        {
            get { return m_position; }
            set
            {
                m_position = value;
                Invalidate();
            }
        }
        public float Radius
        {
            get { return m_radius; }
            set
            {
                m_radius = value;
                Invalidate();
            }
        }
        public Color Color
        {
            get { return m_color; }
            set
            {
                m_color = value;
                Invalidate();
            }
        }

        public void Draw(Graphics graphics)
        {
            Update();
            graphics.FillPath(m_brush, m_path);
        }
        /// <summary>
        /// Calculate a Bresenham line between two points
        /// </summary>
        /// <param name="p0">start point</param>
        /// <param name="p1">end point</param>
        /// <returns>generic list of points starting at p0 to p1</returns>
        private List<Point> line(Point p0, Point p1)
        {
            List<Point> l = new List<Point>();

            Boolean steep = Math.Abs(p1.Y - p0.Y) > Math.Abs(p1.X - p0.X);

            if (steep == true)
            {
                Point tmpPoint = new Point(p0.X, p0.Y);
                p0 = new Point(tmpPoint.Y, tmpPoint.X);

                tmpPoint = p1;
                p1 = new Point(tmpPoint.Y, tmpPoint.X);
            }

            int deltaX = Math.Abs(p1.X - p0.X);
            int deltaY = Math.Abs(p1.Y - p0.Y);
            int error = 0;
            int deltaError = deltaY;
            int yStep = 0;
            int xStep = 0;
            int y = p0.Y;
            int x = p0.X;

            if (p0.Y < p1.Y)
            {
                yStep = 1;
            }
            else
            {
                yStep = -1;
            }

            if (p0.X < p1.X)
            {
                xStep = 1;
            }
            else
            {
                xStep = -1;
            }

            int tmpX = 0;
            int tmpY = 0;

            while (x != p1.X)
            {

                x += xStep;
                error += deltaError;

                //if the error exceeds the X delta then
                //move one along on the Y axis
                if ((2 * error) > deltaX)
                {
                    y += yStep;
                    error -= deltaX;
                }

                //flip the coords if they're steep
                if (steep)
                {
                    tmpX = y;
                    tmpY = x;
                }
                else
                {
                    tmpX = x;
                    tmpY = y;
                }

                //check the point generated is legal
                //and if it is add it to the list
                if (m_lightingLayer.CheckPoint(tmpX, tmpY) == true)
                {
                    l.Add(new Point(tmpX, tmpY));
                }
                else
                {   //a bad point has been found, so return the list thus far
                    return l;
                }

            }

            return l;
        }

        public void Invalidate()
        {
            m_invalid = true;
        }
    }
}