﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using RPGOnline.Controls.Map.Events;
using RPGOnline.Controls.Map.Layers;
using RPGOnline.Properties;

namespace RPGOnline.Controls.Map
{
    public partial class MapDrawerControl : UserControl
    {
        private Map m_map;
    	private Device m_device;
    	private SwapChain m_swapChain;
    	private Surface m_backBuffer;
    	private Surface m_depthStencilSurface;
        private BackgroundLayer m_backgroundLayer;
        private GridLayer m_gridLayer;
        public event EventHandler<CellEventArgs> OnCellMouseDown;
        public event EventHandler<CellEventArgs> OnCellMouseMove;
        public event EventHandler<CellEventArgs> OnCellMouseUp;

        public MapDrawerControl()
        {
            SetStyle(ControlStyles.UserPaint, true);
            SetStyle(ControlStyles.DoubleBuffer, false);
            SetStyle(ControlStyles.OptimizedDoubleBuffer, false);
            SetStyle(ControlStyles.ResizeRedraw, false);
            SetStyle(ControlStyles.AllPaintingInWmPaint, true);
        	SetStyle(ControlStyles.Opaque, true);
			SetStyle(ControlStyles.SupportsTransparentBackColor, false);

            InitializeComponent();

			CreateDevice();
			CreateSwapChain();
        }

        public Map Map 
        { 
            get
            {
                return m_map;
            } 
            set
            {
                m_map = value;
                if (m_map == null)
                    return;

                Width = m_map.MapSize.Width*m_map.TileSize.Width;
                Height = m_map.MapSize.Height*m_map.TileSize.Height;

            	CreateSwapChain();

                m_backgroundLayer = new BackgroundLayer(m_device, m_map);
                m_gridLayer = new GridLayer(m_device, m_map);
            } 
        }

        private void CreateDevice()
		{
			PresentParameters presentParameters = GetPresentParameters();

			m_device = new Device(0, DeviceType.Hardware, this, CreateFlags.HardwareVertexProcessing, presentParameters);
			m_device.DeviceLost += OnDeviceLost;

			CreateSwapChain();
		}

    	private PresentParameters GetPresentParameters()
    	{
    		PresentParameters presentParameters = new PresentParameters();
    		presentParameters.AutoDepthStencilFormat = DepthFormat.D24X8;
    		presentParameters.BackBufferCount = 1;
    		presentParameters.BackBufferFormat = Format.A8R8G8B8;
    		presentParameters.BackBufferWidth = Width;
    		presentParameters.BackBufferHeight = Height;
    		presentParameters.SwapEffect = SwapEffect.Flip;
    		presentParameters.Windowed = true;
    		presentParameters.DeviceWindow = this;
    		presentParameters.PresentationInterval = PresentInterval.Immediate;
    		return presentParameters;
    	}

    	private void CreateSwapChain()
    	{
    		PresentParameters presentParameters = GetPresentParameters();

    		m_swapChain = new SwapChain(m_device, presentParameters);
    		
			m_backBuffer = m_swapChain.GetBackBuffer(0, BackBufferType.Mono);
    		m_depthStencilSurface = m_device.CreateDepthStencilSurface(m_backBuffer.Description.Width, m_backBuffer.Description.Height, DepthFormat.D24X8,
    		                                                           MultiSampleType.None, 0, false);

    		m_device.SetRenderTarget(0, m_backBuffer);
    		m_device.DepthStencilSurface = m_depthStencilSurface;
			m_device.Viewport = new Viewport() { X = 0, Y = 0, Width = Width, Height = Height, MinZ = 0, MaxZ = 1 };
    	}

    	private void OnDeviceLost(object sender, EventArgs e)
    	{
			m_backBuffer.Dispose();
			m_depthStencilSurface.Dispose();
    		m_swapChain.Dispose();
    	}

    	public MapLayer OverlayLayer { get { return m_map.OverlayLayer; } }

        [Browsable(false)]
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        public bool GridVisible
        {
            get { return m_gridLayer.Visible; }
            set { m_gridLayer.Visible = value; }
        }

        public Device Device
        {
            get { return m_device; }
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            if (m_map == null)
                return;

    		m_device.Transform.Projection = Matrix.OrthoOffCenterLH(0, Width, 0, Height, 0, 1);
    		m_device.Transform.World = Matrix.Identity;
    		m_device.Transform.View = Matrix.Identity;
    		m_device.RenderState.ZBufferEnable = false;

            m_device.RenderState.SourceBlend = Blend.SourceAlpha;
            m_device.RenderState.DestinationBlend = Blend.InvSourceAlpha;

            m_device.TextureState[0].ColorOperation = TextureOperation.SelectArg1;
            m_device.TextureState[0].ColorArgument1 = TextureArgument.TextureColor;
            m_device.TextureState[0].ColorArgument2 = TextureArgument.Diffuse;

            m_device.TextureState[0].AlphaOperation = TextureOperation.Modulate;
            m_device.TextureState[0].AlphaArgument1 = TextureArgument.Diffuse;
            m_device.TextureState[0].AlphaArgument2 = TextureArgument.TextureColor;

			m_device.BeginScene();
			m_device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Gray, 1, 0);

            m_backgroundLayer.Draw();
            m_map.Draw();
            m_gridLayer.Draw();

			m_device.EndScene();
			m_swapChain.Present();
            
            /*m_map.Draw(e.Graphics);
            DrawGrid(e);*/
        }

    	protected override void OnMouseDown(MouseEventArgs e)
        {
            base.OnMouseDown(e);

            if (m_map == null)
                return;

            int x = e.X/m_map.TileSize.Width;
            int y = e.Y/m_map.TileSize.Height;

            if(x < 0 || x >= m_map.MapSize.Width || y < 0 || y >= m_map.MapSize.Height)
                return;

            if (OnCellMouseDown != null)
                OnCellMouseDown(this, new CellEventArgs(e, new Point(x,y)));
        }

        protected override void OnMouseMove(MouseEventArgs e)
        {
            base.OnMouseMove(e);

            if (m_map == null)
                return;

            int x = e.X / m_map.TileSize.Width;
            int y = e.Y / m_map.TileSize.Height;

            if (x < 0 || x >= m_map.MapSize.Width || y < 0 || y >= m_map.MapSize.Height)
                return;

            if (OnCellMouseMove != null)
                OnCellMouseMove(this, new CellEventArgs(e, new Point(x, y)));
        }

        protected override void OnMouseUp(MouseEventArgs e)
        {
            base.OnMouseUp(e);

            if (m_map == null)
                return;

            int x = e.X / m_map.TileSize.Width;
            int y = e.Y / m_map.TileSize.Height;

            if (x < 0 || x >= m_map.MapSize.Width || y < 0 || y >= m_map.MapSize.Height)
                return;

            if (OnCellMouseUp != null)
                OnCellMouseUp(this, new CellEventArgs(e, new Point(x, y)));
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            Invalidate();
        }
    }
}
