using System;
using System.Drawing;
using System.Drawing.Imaging;
using Microsoft.DirectX.Direct3D;

namespace RPGOnline.Controls.Map
{
    public class MapTileset
    {
        private readonly Device m_device;
        private readonly Bitmap m_tileSetImage;
        private readonly Size m_tileSize;

        private MapCell[,] m_tiles;
        private Texture m_texture;

        public MapCell[,] Tiles { get { return m_tiles; } }
        public int Width { get; private set; }
        public int Height { get; private set; }
        public int PixelWidth { get { return Width*m_tileSize.Width; } }
        public int PixelHeight { get { return Height*m_tileSize.Height; } }
        public Size TileSize { get { return m_tileSize; } }

        public MapTileset(Device device, Bitmap tileSetImage, Size tileSize, int framesPerTile = 1)
        {
            m_device = device;
            m_tileSetImage = tileSetImage;
            m_tileSize = tileSize;

            Width = (tileSetImage.Width/tileSize.Width)/framesPerTile;
            Height = (tileSetImage.Height/tileSize.Height);

            m_texture = new Texture(m_device, tileSetImage, Usage.None, Pool.Managed);
            m_tiles = new MapCell[Width,Height];

            for(int x = 0; x < Width; x++)
            {
                for(int y = 0; y < Height; y++)
                {
                    m_tiles[x, y] = new MapCell();

                    for (int f = 0; f < framesPerTile; f++)
                    {
                        m_tiles[x,y].Frames.Add(
                            new MapCellFrame(
                                new Point((x + Width * f) * tileSize.Width, y * tileSize.Height), 
                                this,
                                m_texture
                            )
                        );
                    }
                }
            }
        }

        public void Compose(MapTileset tileset)
        {
            if (!m_tileSize.Equals(tileset.TileSize))
                return;

            Bitmap transparentBitmap = new Bitmap(m_tileSize.Width, m_tileSize.Height, PixelFormat.Format32bppArgb);
            transparentBitmap.MakeTransparent();

            MapCell[,] newTiles = new MapCell[Width + tileset.Width, Math.Max(Height, tileset.Height)];
            for (int x = 0; x < Width + tileset.Width; x++)
                for (int y = 0; y < Math.Max(Height, tileset.Height); y++)
                {
                    newTiles[x, y] = new MapCell();
                    //newTiles[x,y].Frames.Add(transparentBitmap);
                }

            for (int x = 0; x < Width; x++)
                for (int y = 0; y < Height; y++)
                    newTiles[x, y] = m_tiles[x, y];

            for (int x = 0; x < tileset.Width; x++)
                for (int y = 0; y < tileset.Height; y++)
                    newTiles[Width + x, y] = tileset.Tiles[x, y];

            m_tiles = newTiles;
            Width += tileset.Width;
            Height = Math.Max(Height, tileset.Height);
        }

        public void Draw(Graphics graphics)
        {
            for (int x = 0; x < Width; x++)
            {
                for (int y = 0; y < Height; y++)
                {
                    Rectangle destinationRectangle = new Rectangle(x * m_tileSize.Width, y * m_tileSize.Height, m_tileSize.Width, m_tileSize.Height);
                    MapCell tile = m_tiles[x, y];

                    if (tile.Frame == null)
                        return;

                    graphics.DrawImage(m_tileSetImage, destinationRectangle, destinationRectangle, GraphicsUnit.Pixel);
                }
            }
        }
    }
}